using DataExportHO.Controller;
using System;
using System.Data;
using System.IO;
using System.Windows.Forms;
using System.Text;
using DataExportHO.Storage;
using System.Xml;


namespace CZ100210
{
    public class DataExportCustomisation : IDataExportExtension
    
        /* 24-06-2017
         * Pre-Requisites - Add se100210.exe & Shoprlib.dll as references
         * GST dependent ledger creation
         * Effective date based Stock items GST tax value extraction
         * HSN code configuration
         * Vendor GSTIN number inclusion as a part of purchase XML 
         * */

        /* 30-06-2017
         * To integrate the dll you need to run the following commands on shoper HO application database
         * We suggest to understand the Shoper HO customization framework and incorporate the changes
         * 
if not exists (Select * from GenLookUp where Recid = 14400 and Code like 'SE100210+DATAEXPORT%' and Descr = 'CZ100210.DataExportCustomisation')
Insert Into GenLookUp (Recid,Code,Descr,Flag) Values (14400,'SE100210+DATAEXPORT+1','CZ100210.DataExportCustomisation',1)


INSERT INTO [ExtensionConfigInternal]	([RecId],[ProgramId],[EventId],
[SrlNo],[DLLName],[ActiveFlag]) VALUES (14400,'TallyControllerExtension','CZ100210.DataExportCustomisation',1,'D:\Shoper9HO\CZ100210.dll',1)
                     
         */
        /*
         * Issues fixed 6.0  - 24/06/2017
         * To extract item specific tax rates as per the sales tax cat / sales tax revision and sales tax revision 
         * 
         * Issues fixed 6.0.1 - 30/06/2017
         * - HSN code tag specific changes done as per 6.0.1 release         
         */
    {
        private ExtensionParams parameters;
        private TallyControllerEventManager eventManager;
        public object GetEventManager()
        {
            return this.eventManager;
        }
        public void Initialize(ExtensionParams extensionParams)
        {
            this.parameters = extensionParams;
            this.eventManager = new TallyControllerEventManager();
        }

        // Following function used to subscribe the events from Data export event controller
        public void Subscribe()
        {
            this.eventManager.EnterProcess += new TallyControllerEventManager.EventHandler(this.eventManager_GSTFileCreationEnterProcess);
            this.eventManager.OPXmlClosing += new TallyControllerEventManager.EventHandler(this.eventManager_OPXmlClosing);
            this.eventManager.StockItemXmlClosing += new TallyControllerEventManager.EventHandler(eventManager_StockItemXmlClosing);
        }

        // Following event is used to edit the Optional export pruchase to make GST compatible
        private void eventManager_OPXmlClosing(object sender, TallyControllerEventArgs e)
        {

            string val = e.DataRow["SuppCode"].ToString();
            string Country = "";
            String State = "";
            String GSTINno = "";

            StringBuilder queryBuilder = new StringBuilder();
            queryBuilder.Append("Select G.GLCountry,G.GLState,S.CSTNumber from geoloc as G  join (Select * from ShowroomMaster where ShoperCompCode = '" + val + "') as S on g.GlCountryCode =S.GLCountry and g.GlStateCode=S.GLState and g.GLCitycode = S.GLCity");

            DataTable data_table = new DataTable();
            DataRow data_row = null;
            data_table = parameters.Connection.ExecuteQueryReturnDataTable(queryBuilder.ToString());
            if (data_table != null)
            {
                if (data_table.Rows.Count > 0)
                {
                    data_row = data_table.Rows[0];
                    Country = data_row["GLCountry"].ToString();
                    State = data_row["GLState"].ToString();
                    GSTINno = data_row["CSTNumber"].ToString();

                }
            }

            
            StringBuilder pxml = e.StringBuilder;
            /*
             * Use the following code to change the purchase ledger name to any specific ledger type in OptionalPurchase_XML
             //  pxml.Replace("<VOUCHERTYPENAME>Purchase</VOUCHERTYPENAME>", "<VOUCHERTYPENAME>Purchase GST</VOUCHERTYPENAME>");
             */

            //MessageBox.Show(this.parameters.ShowroomInfo.ShrmFromDate + " : T file containg purchase transactions");
            //By default the vendor type is sent as Regular
            pxml.Append("<STATENAME>" + State + "</STATENAME><GSTREGISTRATIONTYPE>Regular</GSTREGISTRATIONTYPE><VATDEALERTYPE>Regular</VATDEALERTYPE><PARTYGSTIN>" + GSTINno + "</PARTYGSTIN>");
            e.StringBuilder = pxml;


           
           
        }

        // Following event is called to export the GST supported ledger formats
        private void eventManager_GSTFileCreationEnterProcess(object sender, TallyControllerEventArgs e)
        {

            StringBuilder builder = new StringBuilder();
            GenerateLedgerXml(builder, "Purchase", "Purchase Accounts");
            GenerateLedgerXml(builder, "Sales", "Sales Accounts");
            GenerateLedgerXml(builder, "CGST", "Duties &amp; Taxes");
            GenerateLedgerXml(builder, "SGST", "Duties &amp; Taxes");
            GenerateLedgerXml(builder, "UTGST", "Duties &amp; Taxes");
            GenerateLedgerXml(builder, "IGST", "Duties &amp; Taxes");
            GenerateLedgerXml(builder, "CESS", "Duties &amp; Taxes");
            e.XmlFileBuilder.FileType = "M";
            e.XmlFileBuilder.StartNewFile();
            e.XmlFileBuilder.GenerateFile(builder.ToString());
        }

        // Following event is used to make the Stock item xml GST compatible
        void eventManager_StockItemXmlClosing(object sender, TallyControllerEventArgs e)
        {
            String val = e.DataRow["ProdTaxType"].ToString();
            String showroom_code = "";
            showroom_code= this.parameters.ShowroomInfo.ShowroomCode;
            DataTable data_table_1 = new DataTable();
            String Tax_Sh_code = "";
            StringBuilder queryBuilder_1 = new StringBuilder();
            queryBuilder_1.Append("Select  GLCountry+'    '+GLZone+'     '+GLState+'    '+'99'    as Loc_state ,");
            queryBuilder_1.Append("GLCountry+'    '+GLZone+'    '+GLState+'     '+GLCity+'    1'  as Loc_city from showroommaster where ShowroomCode = '" + showroom_code + "'");
            data_table_1 = parameters.Connection.ExecuteQueryReturnDataTable(queryBuilder_1.ToString());


            if ((data_table_1 != null & data_table_1.Rows.Count > 0))
            {
                foreach (DataRow data_row_1 in data_table_1.Rows)
                {
                    Tax_Sh_code = data_row_1["Loc_state"].ToString();
                }
            }

            
            StringBuilder builder = e.StringBuilder;
            String HSNno = "";
            String HSNdesc = "";
  

            // You can use different analysis codes of itemmaster to store and send the HSN information
            // 1 - Enable specific analysis codes in Shoper HO system parameters 
            // 2 - Catalouge HSN codes in genlookup.
            // 3 - Capture the HSN codes in Itemmaster for stock items
            if (e.DataRow != null)
            {
                HSNno = e.DataRow["Analysis1cd"].ToString();
                HSNdesc = e.DataRow["Analysis2cd"].ToString();
            }


            //<VATAPPLICABLE>&#4; Applicable</VATAPPLICABLE>
            builder.Append("<GSTAPPLICABLE>&#4; Applicable</GSTAPPLICABLE><GSTTYPEOFSUPPLY>Goods</GSTTYPEOFSUPPLY>");
         //   builder.Append("<HSNCODE>" + HSNno + "</HSNCODE><HSN>" + HSNdesc + "</HSN>");

            DataTable data_table = new DataTable();
            
            StringBuilder queryBuilder = new StringBuilder();
         
            StringBuilder Taxdata = new StringBuilder();

            //Following query is enhanced to extract tax rates with effective dates.
            // If only CGST and SGST are defined, it will add both to get the IGST value
            queryBuilder.Append("Select distinct Convert(VARCHAR, EffectiveDate, 112) as EffectiveDate,T1Rate,T2Rate,ISNULL(T3Rate,0) as T3Rate,ISNULL(T1Rate,0)+ISNULL(T2Rate,0) as IGST from salestaxcat  where EffectiveDate >= '2017-07-01 00:00:00.000' and prodtaxtype = '" + val + "' and DestTaxType =0  and ShowRoomCode ='"+Tax_Sh_code+"'");
            queryBuilder.Append("Union Select distinct Convert(VARCHAR, EffectiveDate, 112) as EffectiveDate,T1Rate,T2Rate,ISNULL(T3Rate,0) as T3Rate,ISNULL(T1Rate,0)+ISNULL(T2Rate,0) as IGST from SalesTaxRevision where EffectiveDate >= '2017-07-01 00:00:00.000' and prodtaxtype = '" + val + "' and DestTaxType =0 and ShowRoomCode ='"+Tax_Sh_code+"'");
            queryBuilder.Append("Union Select  distinct Convert(VARCHAR, EffectiveDate, 112) as EffectiveDate,T1Rate,T2Rate,ISNULL(T3Rate,0) as T3Rate,ISNULL(T1Rate,0)+ISNULL(T2Rate,0) as IGST from SalesTaxRevisionHistory  where EffectiveDate >= '2017-07-01 00:00:00.000' and prodtaxtype = '" + val + "' and DestTaxType =0 and ShowRoomCode ='"+Tax_Sh_code+"'");

            data_table = parameters.Connection.ExecuteQueryReturnDataTable(queryBuilder.ToString());
            
                        
            if ((data_table != null & data_table.Rows.Count > 0) )
            {
                foreach (DataRow data_row in data_table.Rows)
                {
                    String Efft_dt = data_row["EffectiveDate"].ToString();
                    String CGST = data_row["T1Rate"].ToString();
                    String SGST = data_row["T2Rate"].ToString();
                    String IGST = data_row["IGST"].ToString();
                    String CESS = data_row["T3Rate"].ToString();
                    if (!(string.IsNullOrEmpty(Efft_dt) & string.IsNullOrEmpty(CGST) & string.IsNullOrEmpty(SGST)))
                    {
                        Taxdata.Append("<GSTDETAILS.LIST><APPLICABLEFROM>" + Efft_dt + "</APPLICABLEFROM><CALCULATIONTYPE>OnValue</CALCULATIONTYPE><HSNMASTERNAME/>");
                        //<HSNCODE>HS0001</HSNCODE><HSNMASTERNAME/><HSN>MYHSNcode</HSN>
                        Taxdata.Append("<HSNCODE>" + HSNno + "</HSNCODE><HSNMASTERNAME/><HSN>" + HSNdesc + "</HSN>");
                        Taxdata.Append("<TAXABILITY>Taxable</TAXABILITY><ISREVERSECHARGEAPPLICABLE>No</ISREVERSECHARGEAPPLICABLE><ISNONGSTGOODS>No</ISNONGSTGOODS>");
                        Taxdata.Append("<GSTINELIGIBLEITC>No</GSTINELIGIBLEITC><STATEWISEDETAILS.LIST><STATENAME>&#4;Any</STATENAME>");
                        Taxdata.Append("<RATEDETAILS.LIST><GSTRATEDUTYHEAD>CentralTax</GSTRATEDUTYHEAD><GSTRATEVALUATIONTYPE>BasedonValue</GSTRATEVALUATIONTYPE><GSTRATE>" + CGST + "</GSTRATE></RATEDETAILS.LIST>");
                        Taxdata.Append("<RATEDETAILS.LIST><GSTRATEDUTYHEAD>StateTax</GSTRATEDUTYHEAD><GSTRATEVALUATIONTYPE>BasedonValue</GSTRATEVALUATIONTYPE><GSTRATE>" + SGST + "</GSTRATE></RATEDETAILS.LIST>");
                        Taxdata.Append("<RATEDETAILS.LIST><GSTRATEDUTYHEAD>IntegratedTax</GSTRATEDUTYHEAD><GSTRATEVALUATIONTYPE>BasedonValue</GSTRATEVALUATIONTYPE><GSTRATE>" + IGST + "</GSTRATE></RATEDETAILS.LIST>");
                        Taxdata.Append("<RATEDETAILS.LIST><GSTRATEDUTYHEAD>Cess</GSTRATEDUTYHEAD><GSTRATEVALUATIONTYPE>BasedonValue</GSTRATEVALUATIONTYPE><GSTRATE>" + CESS + "</GSTRATE></RATEDETAILS.LIST>");
                        Taxdata.Append("<GSTSLABRATES.LIST></GSTSLABRATES.LIST></STATEWISEDETAILS.LIST><TEMPGSTDETAILSLABRATES.LIST></TEMPGSTDETAILSLABRATES.LIST></GSTDETAILS.LIST>");
                    }
                }

                builder.Append(Taxdata);
            }

        }


        //Supporting functions to generate the ledgers
        private void GenerateLedgerXml(StringBuilder builder, string ledgerName, string parent)
        {
            builder.Append("<TALLYMESSAGE xmlns:UDF=\"TallyUDF\">");
            builder.Append("<LEDGER NAME=\"").Append(ledgerName).Append("\" RESERVEDNAME=\"\">");
            builder.Append("<PARENT>").Append(parent).Append("</PARENT>");
            builder.Append("<TAXTYPE>Others</TAXTYPE>");
            builder.Append("<GSTAPPLICABLE>&#4; Applicable</GSTAPPLICABLE>");
            builder.Append("<VATAPPLICABLE> &#4; Applicable</VATAPPLICABLE>");

            if (ledgerName.Contains("CGST"))
            {
                builder.Append("<TAXTYPE>GST</TAXTYPE><GSTDUTYHEAD>Central Tax</GSTDUTYHEAD>");
            }

            else if (ledgerName.Contains("SGST"))
            {
                builder.Append("<TAXTYPE>GST</TAXTYPE><GSTDUTYHEAD>State Tax</GSTDUTYHEAD>");
            }

            else if (ledgerName.Contains("IGST"))
            {
                builder.Append("<TAXTYPE>GST</TAXTYPE><GSTDUTYHEAD>Integrated Tax</GSTDUTYHEAD>");
            }

            else if (ledgerName.Contains("CESS"))
            {
                builder.Append("<TAXTYPE>GST</TAXTYPE><GSTDUTYHEAD>Cess</GSTDUTYHEAD>");
            }
            else if (ledgerName.Contains("UTGST"))
            {
                builder.Append("<TAXTYPE>GST</TAXTYPE><GSTDUTYHEAD>State Tax</GSTDUTYHEAD>");
            }



            builder.Append("<LANGUAGENAME.LIST>");
            builder.Append("<NAME.LIST TYPE=\"String\">");
            builder.Append("<ISEXCISEAPPLICABLE>Yes</ISEXCISEAPPLICABLE>");
            builder.Append("<EXCISEALLOCTYPE>Based on Value</EXCISEALLOCTYPE>");
            builder.Append("<NAME>").Append(ledgerName).Append("</NAME>");
            builder.Append("</NAME.LIST>");
            builder.Append("<LANGUAGEID>1033</LANGUAGEID>");
            builder.Append("</LANGUAGENAME.LIST>");
            //builder.Append("<>");
            builder.Append(this.GetShoperInfoUDF(false));
            builder.Append("</LEDGER>");
            builder.Append("</TALLYMESSAGE>");
        }
       // Following fucntion used to generate the UDF values
        private string GetShoperInfoUDF(bool alteredByTag)
        {
            StringBuilder builder = new StringBuilder();
            if (alteredByTag)
            {
                builder.Append("<ALTEREDBY>ShoperHO</ALTEREDBY>");
            }
            XmlFileBuilder.ConstructUDF(builder, "TPIsFromShoper", "Logical", "Yes");
            XmlFileBuilder.ConstructUDF(builder, "TPShoperHOCode", "String", HOInfo.SourceCompCode);
            XmlFileBuilder.ConstructUDF(builder, "TPPrincipalID", "String", HOInfo.GuidInfo);
            return builder.ToString();
        }
    }
}